/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <utmp.h>
#include <string.h>
#include "h.h"

static char buf[BUFSIZE];


/*
** m_functions execute protocol messages on this server:
**
**	cptr	is always NON-NULL, pointing to a *LOCAL* client
**		structure (with an open socket connected!). This
**		identifies the physical socket where the message
**		originated (or which caused the m_function to be
**		executed--some m_functions may call others...).
**
**	sptr	is the source of the message, defined by the
**		prefix part of the message if present. If not
**		or prefix not found, then sptr==cptr.
**
**		(!IsServer(cptr)) => (cptr == sptr), because
**		prefixes are taken *only* from servers...
**
**		(IsServer(cptr))
**			(sptr == cptr) => the message didn't
**			have the prefix.
**
**			(sptr != cptr && IsServer(sptr) means
**			the prefix specified servername. (?)
**
**			(sptr != cptr && !IsServer(sptr) means
**			that message originated from a remote
**			user (not local).
**
**		combining
**
**		(!IsServer(sptr)) means that, sptr can safely
**		taken as defining the target structure of the
**		message in this server.
**	*Always* true (if 'parse' and others are working correct):
**
**	1)	sptr->from == cptr  (note: cptr->from == cptr)
**
**	2)	MyConnect(sptr) <=> sptr == cptr (e.g. sptr
**		*cannot* be a local connection, unless it's
**		actually cptr!). [MyConnect(x) should probably
**		be defined as (x == x->from) --msa ]
**
**	parc	number of variable parameter strings (if zero,
**		parv is allowed to be NULL)
**
**	parv	a NULL terminated list of parameter pointers,
**
**			parv[0], sender (prefix string), if not present
**				this points to an empty string.
**			parv[1]...parv[parc-1]
**				pointers to additional parameters
**			parv[parc] == NULL, *always*
**
**		note:	it is guaranteed that parv[0]..parv[parc-1] are all
**			non-NULL pointers.
*/
  /*
     ** m_close - added by Darren Reed Jul 13 1992.
   */
int
m_close (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
  Reg1 aClient *acptr;
  Reg2 int i;
  int closed = 0;

  if (!MyOper (sptr))
    {
      sendto_one (sptr, err_str (ERR_NOPRIVILEGES), me.name, parv[0]);
      return 0;
    }

  for (i = highest_fd; i; i--)
    {
      if (!(acptr = local[i]))
	continue;
      if (!IsUnknown (acptr) && !IsConnecting (acptr) && !IsHandshake (acptr))
	continue;
      sendto_one (sptr, rpl_str (RPL_CLOSING), me.name, parv[0],
		  get_client_name (acptr, TRUE), acptr->status);
      (void) exit_client (cptr, acptr, &me, "Oper Closing");
      closed++;
    }
  sendto_one (sptr, rpl_str (RPL_CLOSEEND), me.name, parv[0], closed);
  return 0;
}

#if defined(OPER_DIE) || defined(LOCOP_DIE)
int
m_die (cptr, sptr, parc, parv)
     aClient *cptr, *sptr;
     int parc;
     char *parv[];
{
  Reg1 aClient *acptr;
  Reg2 int i;

#ifndef	LOCOP_DIE
  if (!MyClient (sptr) || !IsOper (sptr))
#else
# ifdef	OPER_DIE
  if (!MyClient (sptr) || !IsAnOper (sptr))
# else
  if (!MyClient (sptr) || !IsLocOp (sptr))
# endif
#endif
    {
      sendto_one (sptr, err_str (ERR_NOPRIVILEGES), me.name, parv[0]);
      return 0;
    }

  for (i = 0; i <= highest_fd; i++)
    {
      if (!(acptr = local[i]))
	continue;
      if (IsClient (acptr))
	sendto_one (acptr,
		    ":%s NOTICE %s :Server Terminating. %s",
		    me.name, acptr->name, get_client_name (sptr, TRUE));
      else if (IsServer (acptr))
	sendto_one (acptr, ":%s ERROR :Terminated by %s",
		    me.name, get_client_name (sptr, TRUE));
    }
  (void) s_die ();
  return 0;
}
#endif
